@@ -333,19 +333,20 @@ int linphone_inc_new_call(LinphoneCore *lc, eXosip_event_t *ev)
 		barmesg=ortp_strdup_printf("%s %s",tmp,_("is contacting you."));
 		lc->vtable.show(lc);
 		lc->vtable.display_status(lc,barmesg);
-		lc->vtable.inv_recv(lc,tmp);
-		ms_free(barmesg);
-		osip_free(tmp);
-		
-		linphone_call_set_state(lc->call,LCStateRinging);
-		eXosip_lock();
-		eXosip_call_send_answer(ev->tid,180,NULL);
-		eXosip_unlock();
+
 		/* play the ring */
 		if (lc->sound_conf.ring_sndcard!=NULL){
 			ms_message("Starting local ring...");
 			lc->ringstream=ring_start(lc->sound_conf.local_ring,2000,lc->sound_conf.ring_sndcard);
 		}
+		linphone_call_set_state(lc->call,LCStateRinging);
+		eXosip_lock();
+		eXosip_call_send_answer(ev->tid,180,NULL);
+		eXosip_unlock();
+
+		lc->vtable.inv_recv(lc,tmp);
+		ms_free(barmesg);
+		osip_free(tmp);		
 	}else{
 		ms_error("Error during sdp negociation. ");
 		eXosip_lock();
@@ -931,9 +932,8 @@ void linphone_registration_faillure(LinphoneCore *lc, eXosip_event_t *ev){
 	const char *reason=NULL;
 	osip_uri_t *requri=osip_message_get_uri(ev->request);
 	char *ru;
+	LinphoneProxyConfig *cfg;
 	
-	osip_uri_to_str(requri,&ru);
-
 	if (ev->response){
 		status_code=osip_message_get_status_code(ev->response);
 		reason=osip_message_get_reason_phrase(ev->response);
@@ -943,15 +943,20 @@ void linphone_registration_faillure(LinphoneCore *lc, eXosip_event_t *ev){
 		case 407:
 			linphone_process_authentication(lc,ev);
 			break;
-		case 403:
-			linphone_proxy_config_process_authentication_failure(lc,ev);
 		default:
+			cfg=linphone_core_get_proxy_config_from_rid(lc,ev->rid);
+			/* if contact is up to date, process the failure, otherwise resend a new register with
+				updated contact first, just in case the faillure is due to incorrect contact */
+			if (linphone_proxy_config_register_again_with_updated_contact(cfg,ev->request,ev->response))
+				return; /*we are retrying with an updated contact*/
+			if (status_code==403) linphone_proxy_config_process_authentication_failure(lc,ev);
+			osip_uri_to_str(requri,&ru);
 			msg=ortp_strdup_printf(_("Registration on %s failed: %s"),ru,(reason!=NULL) ? reason : _("no response timeout"));
 			lc->vtable.display_status(lc,msg);
 			gstate_new_state(lc, GSTATE_REG_FAILED, msg);
 			ms_free(msg);
+			osip_free(ru);
 	}
-	osip_free(ru);
 }
 
 void linphone_registration_success(LinphoneCore *lc,eXosip_event_t *ev){
@@ -1001,22 +1006,31 @@ static void linphone_other_request(LinphoneCore *lc, eXosip_event_t *ev){
 		linphone_core_text_received(lc,ev);
 		eXosip_message_send_answer(ev->tid,200,NULL);
 	}else if (strcmp(ev->request->sip_method,"OPTIONS")==0){
-#if 1
 		osip_message_t *options=NULL;
 		eXosip_options_build_answer(ev->tid,200,&options);
 		osip_message_set_allow(options,"INVITE, ACK, BYE, CANCEL, OPTIONS, MESSAGE, SUBSCRIBE, NOTIFY, INFO");
 		osip_message_set_accept(options,"application/sdp");
 		eXosip_options_send_answer(ev->tid,200,options);
-#else
-		ms_warning("Not answering to this options request.");
-#endif
 	}else if (strcmp(ev->request->sip_method,"WAKEUP")==0
 		&& comes_from_local_if(ev->request)) {
 		eXosip_message_send_answer(ev->tid,200,NULL);
 		ms_message("Receiving WAKEUP request !");
 		if (lc->vtable.show)
 			lc->vtable.show(lc);
-	}else {
+	}else if (strncmp(ev->request->sip_method, "REFER", 5) == 0){
+		ms_message("Receiving REFER request !");
+		if (comes_from_local_if(ev->request)) {
+			osip_header_t *h=NULL;
+			osip_message_header_get_byname(ev->request,"Refer-To",0,&h);
+			eXosip_message_send_answer(ev->tid,200,NULL);
+			if (h){
+				if (lc->vtable.refer_received) 
+					lc->vtable.refer_received(lc,h->hvalue);
+			}
+			
+		}else ms_warning("Ignored REFER not coming from this local loopback interface.");
+	}
+    	else {
 		char *tmp=NULL;
 		size_t msglen=0;
 		osip_message_to_str(ev->request,&tmp,&msglen);
