@@ -23,6 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "private.h"
 #include "mediastreamer2/mediastream.h"
 #include "mediastreamer2/msvolume.h"
+#include "mediastreamer2/msequalizer.h"
 #include <eXosip2/eXosip.h>
 #include "sdphandler.h"
 
@@ -78,6 +79,7 @@ int lc_callback_obj_invoke(LCCallbackObj *obj, LinphoneCore *lc){
 static void  linphone_call_init_common(LinphoneCall *call, char *from, char *to){
 	call->state=LCStateInit;
 	call->start_time=time(NULL);
+	call->media_start_time=0;
 	call->log=linphone_call_log_new(call, from, to);
 	linphone_core_notify_all_friends(call->core,LINPHONE_STATUS_ONTHEPHONE);
 	if (linphone_core_get_firewall_policy(call->core)==LINPHONE_POLICY_USE_STUN) 
@@ -106,19 +108,18 @@ static void discover_mtu(LinphoneCore *lc, const char *remote){
 LinphoneCall * linphone_call_new_outgoing(struct _LinphoneCore *lc, const osip_from_t *from, const osip_to_t *to)
 {
 	LinphoneCall *call=ms_new0(LinphoneCall,1);
-	char localip[LINPHONE_IPADDR_SIZE];
 	char *fromstr=NULL,*tostr=NULL;
 	call->dir=LinphoneCallOutgoing;
 	call->cid=-1;
 	call->did=-1;
 	call->tid=-1;
 	call->core=lc;
-	linphone_core_get_local_ip(lc,to->url->host,localip);
+	linphone_core_get_local_ip(lc,to->url->host,call->localip);
 	osip_from_to_str(from,&fromstr);
 	osip_to_to_str(to,&tostr);
 	linphone_call_init_common(call,fromstr,tostr);
 	call->sdpctx=sdp_handler_create_context(&linphone_sdphandler,
-		call->audio_params.natd_port>0 ? call->audio_params.natd_addr : localip,
+		call->audio_params.natd_port>0 ? call->audio_params.natd_addr : call->localip,
 		from->url->username,NULL);
 	sdp_context_set_user_pointer(call->sdpctx,(void*)call);
 	discover_mtu(lc,to->url->host);
@@ -126,9 +127,7 @@ LinphoneCall * linphone_call_new_outgoing(struct _LinphoneCore *lc, const osip_f
 }
 
 
-LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, const char *from, const char *to, int cid, int did, int tid)
-{
-	char localip[LINPHONE_IPADDR_SIZE];
+LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, const char *from, const char *to, int cid, int did, int tid){
 	LinphoneCall *call=ms_new0(LinphoneCall,1);
 	osip_from_t *me= linphone_core_get_primary_contact_parsed(lc);
 	osip_from_t *from_url=NULL;
@@ -139,10 +138,10 @@ LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, const char *from, co
 	call->core=lc;
 	osip_from_init(&from_url);
 	osip_from_parse(from_url, from);
-	linphone_core_get_local_ip(lc,from_url->url->host,localip);
+	linphone_core_get_local_ip(lc,from_url->url->host,call->localip);
 	linphone_call_init_common(call, osip_strdup(from), osip_strdup(to));
 	call->sdpctx=sdp_handler_create_context(&linphone_sdphandler,
-		call->audio_params.natd_port>0 ? call->audio_params.natd_addr : localip,
+		call->audio_params.natd_port>0 ? call->audio_params.natd_addr : call->localip,
 		me->url->username,NULL);
 	sdp_context_set_user_pointer(call->sdpctx,(void*)call);
 	discover_mtu(lc,from_url->url->host);
@@ -250,7 +249,14 @@ void linphone_call_log_destroy(LinphoneCallLog *cl){
 int linphone_core_get_current_call_duration(const LinphoneCore *lc){
 	LinphoneCall *call=lc->call;
 	if (call==NULL) return 0;
-	return time(NULL)-call->start_time;
+	if (call->media_start_time==0) return 0;
+	return time(NULL)-call->media_start_time;
+}
+
+const char *linphone_core_get_remote_uri(LinphoneCore *lc){
+	LinphoneCall *call=lc->call;
+	if (call==NULL) return 0;
+	return call->dir==LinphoneCallIncoming ? call->log->from : call->log->to;
 }
 
 void _osip_trace_func(char *fi, int li, osip_trace_level_t level, char *chfr, va_list ap){
@@ -334,17 +340,29 @@ net_config_read (LinphoneCore *lc)
 	linphone_core_set_mtu(lc,tmp);
 }
 
+static void build_sound_devices_table(LinphoneCore *lc){
+	const char **devices;
+	const char **old;
+	int ndev;
+	int i;
+	const MSList *elem=ms_snd_card_manager_get_list(ms_snd_card_manager_get());
+	ndev=ms_list_size(elem);
+	devices=ms_malloc((ndev+1)*sizeof(const char *));
+	for (i=0;elem!=NULL;elem=elem->next,i++){
+		devices[i]=ms_snd_card_get_string_id((MSSndCard *)elem->data);
+	}
+	devices[ndev]=NULL;
+	old=lc->sound_conf.cards;
+	lc->sound_conf.cards=devices;
+	if (old!=NULL) ms_free(old);
+}
 
 void sound_config_read(LinphoneCore *lc)
 {
 	/*int tmp;*/
 	const char *tmpbuf;
 	const char *devid;
-	const MSList *elem;
-	const char **devices;
-	int ndev;
-	int i;
-#ifndef WIN32
+#ifdef __linux
 	/*alsadev let the user use custom alsa device within linphone*/
 	devid=lp_config_get_string(lc->config,"sound","alsadev",NULL);
 	if (devid){
@@ -353,14 +371,8 @@ void sound_config_read(LinphoneCore *lc)
 	}
 #endif
 	/* retrieve all sound devices */
-	elem=ms_snd_card_manager_get_list(ms_snd_card_manager_get());
-	ndev=ms_list_size(elem);
-	devices=ms_malloc((ndev+1)*sizeof(const char *));
-	for (i=0;elem!=NULL;elem=elem->next,i++){
-		devices[i]=ms_snd_card_get_string_id((MSSndCard *)elem->data);
-	}
-	devices[ndev]=NULL;
-	lc->sound_conf.cards=devices;
+	build_sound_devices_table(lc);
+
 	devid=lp_config_get_string(lc->config,"sound","playback_dev_id",NULL);
 	linphone_core_set_playback_device(lc,devid);
 	
@@ -411,6 +423,8 @@ void sound_config_read(LinphoneCore *lc)
 
 	linphone_core_enable_echo_limiter(lc,
 		lp_config_get_int(lc->config,"sound","echolimiter",0));
+	linphone_core_enable_agc(lc,
+		lp_config_get_int(lc->config,"sound","agc",0));
 }
 
 void sip_config_read(LinphoneCore *lc)
@@ -481,6 +495,8 @@ void sip_config_read(LinphoneCore *lc)
 	/*for test*/
 	lc->sip_conf.sdp_200_ack=lp_config_get_int(lc->config,"sip","sdp_200_ack",0);
 	lc->sip_conf.only_one_codec=lp_config_get_int(lc->config,"sip","only_one_codec",0);
+	lc->sip_conf.register_only_when_network_is_up=
+		lp_config_get_int(lc->config,"sip","register_only_when_network_is_up",0);
 }
 
 void rtp_config_read(LinphoneCore *lc)
@@ -643,18 +659,19 @@ const char * linphone_core_get_version(void){
 static PayloadType * payload_type_h264_packetization_mode_1=NULL;
 static PayloadType * linphone_h263_1998=NULL;
 static PayloadType * linphone_mp4v_es=NULL;
+static PayloadType * linphone_h263_old=NULL;
 #endif
 
 void linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, const char *config_path, void * userdata)
 {
 	memset (lc, 0, sizeof (LinphoneCore));
 	lc->data=userdata;
-	
+
 	memcpy(&lc->vtable,vtable,sizeof(LinphoneCoreVTable));
 
-        gstate_initialize(lc);
-        gstate_new_state(lc, GSTATE_POWER_STARTUP, NULL);
-        
+	gstate_initialize(lc);
+	gstate_new_state(lc, GSTATE_POWER_STARTUP, NULL);
+	
 	ortp_init();
 	rtp_profile_set_payload(&av_profile,115,&payload_type_lpc1015);
 	rtp_profile_set_payload(&av_profile,110,&payload_type_speex_nb);
@@ -670,6 +687,10 @@ void linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, co
 	payload_type_set_recv_fmtp(linphone_h263_1998,"CIF=1;QCIF=1");
 	rtp_profile_set_payload(&av_profile,98,linphone_h263_1998);
 
+	linphone_h263_old=payload_type_clone(&payload_type_h263);
+	payload_type_set_recv_fmtp(linphone_h263_old,"QCIF=2");
+ 	rtp_profile_set_payload(&av_profile,34,linphone_h263_old);
+
 	linphone_mp4v_es=payload_type_clone(&payload_type_mp4v);
 	payload_type_set_recv_fmtp(linphone_mp4v_es,"profile-level-id=3");
 	rtp_profile_set_payload(&av_profile,99,linphone_mp4v_es);
@@ -755,30 +776,14 @@ void linphone_core_get_local_ip(LinphoneCore *lc, const char *dest, char *result
 		strncpy(result,linphone_core_get_nat_address(lc),LINPHONE_IPADDR_SIZE);
 		return;
 	}
-	if (linphone_core_get_firewall_policy(lc)==LINPHONE_POLICY_USE_STUN) {
-		if (lc->sip_conf.ipv6_enabled){
-			ms_warning("stun support is not implemented for ipv6");
-		}else{
-			/* we no more use stun for sip socket*/
-#if 0
-			int mport=0;
-			ms_message("doing stun lookup for local address...");
-			if (stun_get_localip(lc,sock,linphone_core_get_sip_port(lc),result,&mport)){
-				if (!lc->net_conf.nat_sdp_only)
-					eXosip_masquerade_contact(result,mport);
-				return;
-			}
-			ms_warning("stun lookup failed, falling back to a local interface...");
-#endif
-		}
-		
-	}
+	if (linphone_core_get_local_ip_for(dest,result)==0)
+		return;
+	/*else fallback to exosip routine that will attempt to find the most realistic interface */
 	if (eXosip_guess_localip(lc->sip_conf.ipv6_enabled ? AF_INET6 : AF_INET,result,LINPHONE_IPADDR_SIZE)<0){
 		/*default to something */
 		strncpy(result,lc->sip_conf.ipv6_enabled ? "::1" : "127.0.0.1",LINPHONE_IPADDR_SIZE);
 		ms_error("Could not find default routable ip address !"); 
-	}	
-	eXosip_masquerade_contact(NULL,0);
+	}
 }
 
 const char *linphone_core_get_primary_contact(LinphoneCore *lc)
@@ -919,7 +924,7 @@ static bool_t exosip_running=FALSE;
 static char _ua_name[64]="Linphone";
 static char _ua_version[64]=LINPHONE_VERSION;
 
-static void set_user_agent(){
+static void apply_user_agent(void){
 	char ua_string[256];
 	snprintf(ua_string,sizeof(ua_string),"%s/%s (eXosip2/%s)",_ua_name,_ua_version,
 #ifdef HAVE_EXOSIP_GET_VERSION
@@ -966,7 +971,7 @@ void linphone_core_set_sip_port(LinphoneCore *lc,int port)
 	eXosip_set_rsvp_mode (lc->rsvp_enable);
 	eXosip_set_rpc_mode (lc->rpc_enable);
 #endif
-	set_user_agent("Linphone",LINPHONE_VERSION);
+	apply_user_agent();
 	exosip_running=TRUE;
 }
 
@@ -996,6 +1001,27 @@ static void linphone_core_disconnected(LinphoneCore *lc){
 	linphone_core_terminate_call(lc,NULL);
 }
 
+static void proxy_update(LinphoneCore *lc, time_t curtime){
+	bool_t doit=FALSE;
+	static time_t last_check=0;
+	static bool_t last_status=FALSE;
+	if (lc->sip_conf.register_only_when_network_is_up){
+		char result[LINPHONE_IPADDR_SIZE];
+		/* only do the network up checking every five seconds */
+		if (last_check==0 || (curtime-last_check)>=5){
+			if (eXosip_guess_localip(lc->sip_conf.ipv6_enabled ? AF_INET6 : AF_INET,result,LINPHONE_IPADDR_SIZE)==0){
+				if (strcmp(result,"::1")!=0 && strcmp(result,"127.0.0.1")!=0){
+					last_status=TRUE;
+					ms_message("Network is up, registering now (%s)",result);
+				}else last_status=FALSE;
+			}
+			last_check=curtime;
+		}
+		doit=last_status;
+	}else doit=TRUE;
+	if (doit) ms_list_for_each(lc->sip_conf.proxies,(void (*)(void*))&linphone_proxy_config_update);
+}
+
 void linphone_core_iterate(LinphoneCore *lc)
 {
 	eXosip_event_t *ev;
@@ -1028,7 +1054,7 @@ void linphone_core_iterate(LinphoneCore *lc)
 		}
 	}
 
-	ms_list_for_each(lc->sip_conf.proxies,(void (*)(void*))&linphone_proxy_config_update);
+	proxy_update(lc,curtime);
 
 	if (lc->call!=NULL){
 		LinphoneCall *call=lc->call;
@@ -1230,6 +1256,57 @@ void linphone_set_sdp(osip_message_t *sip, const char *sdpmesg){
 	osip_message_set_content_length(sip,clen);
 }
 
+LinphoneProxyConfig * linphone_core_lookup_known_proxy(LinphoneCore *lc, const char *uri){
+	const MSList *elem;
+	LinphoneProxyConfig *found_cfg=NULL;
+	osip_from_t *parsed_uri;
+	osip_from_init(&parsed_uri);
+	osip_from_parse(parsed_uri,uri);
+	for (elem=linphone_core_get_proxy_config_list(lc);elem!=NULL;elem=elem->next){
+		LinphoneProxyConfig *cfg=(LinphoneProxyConfig*)elem->data;
+		const char *domain=linphone_proxy_config_get_domain(cfg);
+		if (domain!=NULL && strcmp(domain,parsed_uri->url->host)==0){
+			found_cfg=cfg;
+			break;
+		}
+	}
+	osip_from_free(parsed_uri);
+	return found_cfg;
+}
+
+static void fix_contact(osip_message_t *msg, const char *localip, LinphoneProxyConfig *dest_proxy){
+	osip_contact_t *ctt=NULL;
+	const char *ip=NULL;
+	int port=5060;
+
+	osip_message_get_contact(msg,0,&ctt);
+	if (ctt!=NULL){
+		if (dest_proxy!=NULL){
+			/* if we know the request will go to a known proxy for which we are registered,
+			we can use the same contact address as in the register */
+			linphone_proxy_config_get_contact(dest_proxy,&ip,&port);
+		}else{
+			ip=localip;
+			port=linphone_core_get_sip_port(dest_proxy->lc);
+		}
+		if (ip!=NULL){
+			osip_free(ctt->url->host);
+			ctt->url->host=osip_strdup(ip);
+		}
+		if (port!=0){
+			char tmp[10]={0};
+			char *str;
+			snprintf(tmp,sizeof(tmp)-1,"%i",port);
+			if (ctt->url->port!=NULL)
+				osip_free(ctt->url->port);
+			ctt->url->port=osip_strdup(tmp);
+			osip_contact_to_str(ctt,&str);
+			ms_message("Contact has been fixed to %s",str);
+			osip_free(str);
+		}
+	}
+}
+
 int linphone_core_invite(LinphoneCore *lc, const char *url)
 {
 	char *barmsg;
@@ -1243,6 +1320,7 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 	osip_from_t *parsed_url2=NULL;
 	osip_to_t *real_parsed_url=NULL;
 	char *real_url=NULL;
+	LinphoneProxyConfig *dest_proxy=NULL;
 	
 	if (lc->call!=NULL){
 		lc->vtable.display_warning(lc,_("Sorry, having multiple simultaneous calls is not supported yet !"));
@@ -1256,16 +1334,23 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 		gstate_new_state(lc, GSTATE_CALL_ERROR, NULL);
 		return -1;
 	}
-	if (proxy!=NULL) {
-		from=linphone_proxy_config_get_identity(proxy);
-		
+	dest_proxy=linphone_core_lookup_known_proxy(lc,real_url);
+
+	if (proxy!=dest_proxy && dest_proxy!=NULL) {
+		ms_message("Overriding default proxy setting for this call:");
+		ms_message("The used identity will be %s",linphone_proxy_config_get_identity(dest_proxy));
 	}
+
+	if (dest_proxy!=NULL) 
+		from=linphone_proxy_config_get_identity(dest_proxy);
+	else if (proxy!=NULL)
+		from=linphone_proxy_config_get_identity(proxy);
+
 	/* if no proxy or no identity defined for this proxy, default to primary contact*/
 	if (from==NULL) from=linphone_core_get_primary_contact(lc);
 
 	err=eXosip_call_build_initial_invite(&invite,real_url,from,
 						route,"Phone call");
-
 	if (err<0){
 		ms_warning("Could not build initial invite");
 		goto end;
@@ -1277,6 +1362,11 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 	osip_from_parse(parsed_url2,from);
 	
 	lc->call=linphone_call_new_outgoing(lc,parsed_url2,real_parsed_url);
+	/*try to be best-effort in giving real local or routable contact address,
+	except when the user choosed to override the ipaddress */
+	if (linphone_core_get_firewall_policy(lc)!=LINPHONE_POLICY_USE_NAT_ADDRESS)
+		fix_contact(invite,lc->call->localip,dest_proxy);
+
 	barmsg=ortp_strdup_printf("%s %s", _("Contacting"), real_url);
 	lc->vtable.display_status(lc,barmsg);
 	ms_free(barmsg);
@@ -1428,6 +1518,14 @@ void linphone_core_init_media_streams(LinphoneCore *lc){
 		}
 		
 	}
+	if (linphone_core_echo_cancelation_enabled(lc)){
+		int len,delay,framesize;
+		len=lp_config_get_int(lc->config,"sound","ec_tail_len",0);
+		delay=lp_config_get_int(lc->config,"sound","ec_delay",0);
+		framesize=lp_config_get_int(lc->config,"sound","ec_framesize",0);
+		audio_stream_set_echo_canceler_params(lc->audiostream,len,delay,framesize);
+	}
+	audio_stream_enable_automatic_gain_control(lc->audiostream,linphone_core_agc_enabled(lc));
 #ifdef VIDEO_ENABLED
 	if (lc->video_conf.display || lc->video_conf.capture)
 		lc->videostream=video_stream_new(linphone_core_get_video_port(lc),linphone_core_ipv6_enabled(lc));
@@ -1442,6 +1540,28 @@ static void linphone_core_dtmf_received(RtpSession* s, int dtmf, void* user_data
 		lc->vtable.dtmf_received(lc, dtmf);
 }
 
+static void parametrize_equalizer(LinphoneCore *lc, AudioStream *st){
+	if (st->equalizer){
+		MSFilter *f=st->equalizer;
+		int enabled=lp_config_get_int(lc->config,"sound","eq_active",0);
+		const char *gains=lp_config_get_string(lc->config,"sound","eq_gains",NULL);
+		ms_filter_call_method(f,MS_EQUALIZER_SET_ACTIVE,&enabled);
+		if (enabled){
+			if (gains){
+				do{
+					int bytes;
+					MSEqualizerGain g;
+					if (sscanf(gains,"%f:%f:%f %n",&g.frequency,&g.gain,&g.width,&bytes)==3){
+						ms_message("Read equalizer gains: %f(~%f) --> %f",g.frequency,g.width,g.gain);
+						ms_filter_call_method(f,MS_EQUALIZER_SET_GAIN,&g);
+						gains+=bytes;
+					}else break;
+				}while(1);
+			}
+		}
+	}
+}
+
 static void post_configure_audio_streams(LinphoneCore *lc){
 	AudioStream *st=lc->audiostream;
 	if (st->volrecv && st->volsend){
@@ -1472,6 +1592,7 @@ static void post_configure_audio_streams(LinphoneCore *lc){
 		if (gain!=-1)
 			audio_stream_set_mic_gain(st,gain);
 	}
+	parametrize_equalizer(lc,st);
 	if (lc->vtable.dtmf_received!=NULL){
 		/* replace by our default action*/
 		audio_stream_play_received_dtmfs(lc->audiostream,FALSE);
@@ -1484,6 +1605,9 @@ void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 	const char *tool="linphone-" LINPHONE_VERSION;
 	/* adjust rtp jitter compensation. It must be at least the latency of the sound card */
 	int jitt_comp=MAX(lc->sound_conf.latency,lc->rtp_conf.audio_jitt_comp);
+
+	if (call->media_start_time==0) call->media_start_time=time(NULL);
+
 	char *cname=ortp_strdup_printf("%s@%s",me->url->username,me->url->host);
 	{
 		StreamParams *audio_params=&call->audio_params;
@@ -1620,6 +1744,10 @@ int linphone_core_accept_call(LinphoneCore *lc, const char *url)
 		ms_error("Fail to build answer for call: err=%i",err);
 		return -1;
 	}
+	/*try to be best-effort in giving real local or routable contact address,
+	except when the user choosed to override the ipaddress */
+	if (linphone_core_get_firewall_policy(lc)!=LINPHONE_POLICY_USE_NAT_ADDRESS)
+		fix_contact(msg,call->localip,NULL);
 	/*if a sdp answer is computed, send it, else send an offer */
 	sdpmesg=call->sdpctx->answerstr;
 	if (sdpmesg==NULL){
@@ -1854,6 +1982,7 @@ const char * linphone_core_get_capture_device(LinphoneCore *lc)
 
 /* returns a static array of string describing the sound devices */ 
 const char**  linphone_core_get_sound_devices(LinphoneCore *lc){
+	build_sound_devices_table(lc);
 	return lc->sound_conf.cards;
 }
 
@@ -1944,6 +2073,14 @@ bool_t linphone_core_echo_limiter_enabled(const LinphoneCore *lc){
 	return lc->sound_conf.ea;
 }
 
+void linphone_core_enable_agc(LinphoneCore *lc, bool_t val){
+	lc->sound_conf.agc=val;
+}
+
+bool_t linphone_core_agc_enabled(const LinphoneCore *lc){
+	return lc->sound_conf.agc;
+}
+
 
 void linphone_core_send_dtmf(LinphoneCore *lc,char dtmf)
 {
@@ -2050,14 +2187,22 @@ static void apply_nat_settings(LinphoneCore *lc){
 
 	if (lc->net_conf.firewall_policy==LINPHONE_POLICY_USE_NAT_ADDRESS){
 		if (tmp!=NULL){
-			if (!lc->net_conf.nat_sdp_only)
+			if (!lc->net_conf.nat_sdp_only){
+				eXosip_set_option(EXOSIP_OPT_SET_IPV4_FOR_GATEWAY,tmp);
+				/* the following does not work in all cases */
+				/*
 				eXosip_masquerade_contact(tmp,lc->sip_conf.sip_port);
+				*/
+			}
 			ms_free(tmp);
 		}
-		else 
+		else{
+			eXosip_set_option(EXOSIP_OPT_SET_IPV4_FOR_GATEWAY,NULL);
 			eXosip_masquerade_contact("",0);
+		}
 	}
 	else {
+		eXosip_set_option(EXOSIP_OPT_SET_IPV4_FOR_GATEWAY,NULL);
 		eXosip_masquerade_contact("",0);	
 	}
 }
@@ -2309,13 +2454,14 @@ void linphone_core_set_mtu(LinphoneCore *lc, int mtu){
 	}else ms_set_mtu(0);//use mediastreamer2 default value
 }
 
-void linphone_core_set_waiting_callback(LinphoneCore *lc, LinphoneWaitingCallback cb){
+void linphone_core_set_waiting_callback(LinphoneCore *lc, LinphoneWaitingCallback cb, void *user_context){
 	lc->wait_cb=cb;
+	lc->wait_ctx=user_context;
 }
 
 void linphone_core_start_waiting(LinphoneCore *lc, const char *purpose){
 	if (lc->wait_cb){
-		lc->wait_ctx=lc->wait_cb(lc,NULL,LinphoneWaitingStart,purpose,0);
+		lc->wait_ctx=lc->wait_cb(lc,lc->wait_ctx,LinphoneWaitingStart,purpose,0);
 	}
 }
 
@@ -2363,6 +2509,7 @@ void sip_config_uninit(LinphoneCore *lc)
 	lp_config_set_int(lc->config,"sip","inc_timeout",config->inc_timeout);
 	lp_config_set_int(lc->config,"sip","use_info",config->use_info);
 	lp_config_set_int(lc->config,"sip","use_ipv6",config->ipv6_enabled);
+	lp_config_set_int(lc->config,"sip","register_only_when_network_is_up",config->register_only_when_network_is_up);
 	for(elem=config->proxies,i=0;elem!=NULL;elem=ms_list_next(elem),i++){
 		LinphoneProxyConfig *cfg=(LinphoneProxyConfig*)(elem->data);
 		linphone_proxy_config_write_to_config_file(lc->config,cfg,i);
