@@ -66,6 +66,7 @@ typedef struct sip_config
 	bool_t ipv6_enabled;
 	bool_t sdp_200_ack;
 	bool_t only_one_codec; /*in SDP answers*/
+	bool_t register_only_when_network_is_up;
 } sip_config_t;
 
 typedef struct rtp_config
@@ -107,6 +108,7 @@ typedef struct sound_config
 	char *remote_ring;
 	bool_t ec;
 	bool_t ea;
+	bool_t agc;
 } sound_config_t;
 
 typedef struct codecs_config
@@ -183,10 +185,12 @@ typedef struct _LinphoneCall
 	int cid; /*call id */
 	int did; /*dialog id */
 	int tid; /*last transaction id*/
+	char localip[LINPHONE_IPADDR_SIZE]; /* our best guess for local ipaddress for this call */
 	struct _sdp_context *sdpctx;
-	time_t start_time;
+	time_t start_time; /*time at which the call was initiated*/
+	time_t media_start_time; /*time at which it was accepted, media streams established*/
 	LCState	state;
-	bool_t auth_pending;	
+	bool_t auth_pending;
 } LinphoneCall;
 
 LinphoneCall * linphone_call_new_outgoing(struct _LinphoneCore *lc, const osip_from_t *from, const osip_to_t *to);
@@ -295,6 +299,8 @@ typedef struct _LinphoneProxyConfig
 	char *type;
 	struct _SipSetupContext *ssctx;
 	int auth_failures;
+	char *contact_addr; /* our IP address as seen by the proxy, read from via 's received= parameter*/
+	int contact_port; /*our IP port as seen by the proxy, read from via's rport= parameter */
 	bool_t commit;
 	bool_t reg_sendregister;
 	bool_t registered;
@@ -336,10 +342,13 @@ typedef struct _LinphoneAccountCreator{
 	bool_t succeeded;
 }LinphoneAccountCreator;
 
+LinphoneAccountCreator *linphone_account_creator_new(struct _LinphoneCore *core, const char *type);
 void linphone_account_creator_set_username(LinphoneAccountCreator *obj, const char *username);
 void linphone_account_creator_set_password(LinphoneAccountCreator *obj, const char *password);
 void linphone_account_creator_set_domain(LinphoneAccountCreator *obj, const char *domain);
-int linphone_account_creator_test(LinphoneAccountCreator *obj);
+const char * linphone_account_creator_get_username(LinphoneAccountCreator *obj);
+const char * linphone_account_creator_get_domain(LinphoneAccountCreator *obj);
+int linphone_account_creator_test_existence(LinphoneAccountCreator *obj);
 LinphoneProxyConfig * linphone_account_creator_validate(LinphoneAccountCreator *obj);
 void linphone_account_creator_destroy(LinphoneAccountCreator *obj);
 
@@ -434,6 +443,7 @@ typedef void (*CallLogUpdated)(struct _LinphoneCore *lc, struct _LinphoneCallLog
 typedef void (*TextMessageReceived)(struct _LinphoneCore *lc, LinphoneChatRoom *room, const char *from, const char *message);
 typedef void (*GeneralStateChange)(struct _LinphoneCore *lc, LinphoneGeneralState *gstate);
 typedef void (*DtmfReceived)(struct _LinphoneCore* lc, int dtmf);
+typedef void (*ReferReceived)(struct _LinphoneCore *lc, const char *refer_to);
 
 typedef struct _LinphoneVTable
 {
@@ -456,6 +466,7 @@ typedef struct _LinphoneVTable
 	TextMessageReceived text_received;
 	GeneralStateChange general_state;
 	DtmfReceived dtmf_received;
+	ReferReceived refer_received;
 } LinphoneCoreVTable;
 
 typedef struct _LCCallbackObj
@@ -710,6 +721,9 @@ bool_t linphone_core_echo_cancelation_enabled(LinphoneCore *lc);
 void linphone_core_enable_echo_limiter(LinphoneCore *lc, bool_t val);
 bool_t linphone_core_echo_limiter_enabled(const LinphoneCore *lc);
 
+void linphone_core_enable_agc(LinphoneCore *lc, bool_t val);
+bool_t linphone_core_agc_enabled(const LinphoneCore *lc);
+
 void linphone_core_set_presence_info(LinphoneCore *lc,int minutes_away,const char *contact,LinphoneOnlineStatus os);
 
 LinphoneOnlineStatus linphone_core_get_presence_info(const LinphoneCore *lc);
@@ -763,6 +777,7 @@ void linphone_core_set_record_file(LinphoneCore *lc, const char *file);
 
 gstate_t linphone_core_get_state(const LinphoneCore *lc, gstate_group_t group);
 int linphone_core_get_current_call_duration(const LinphoneCore *lc);
+const char *linphone_core_get_remote_uri(LinphoneCore *lc);
 
 int linphone_core_get_mtu(const LinphoneCore *lc);
 void linphone_core_set_mtu(LinphoneCore *lc, int mtu);
@@ -777,12 +792,14 @@ struct _LpConfig *linphone_core_get_config(LinphoneCore *lc);
 
 /* attempts to wake up another linphone engine already running.
 The "show" callback is called for the other linphone, causing gui to show up.
+call_addr is an optional sip-uri to call immediately after waking up.
 The method returns 0 if an already running linphone was found*/
-int linphone_core_wake_up_possible_already_running_instance(const char *config_file);
+
+int linphone_core_wake_up_possible_already_running_instance(
+    const char * config_file, const char * call_addr);
 
 /*set a callback for some blocking operations, it takes you informed of the progress of the operation*/
-void linphone_core_set_waiting_callback(LinphoneCore *lc, LinphoneWaitingCallback cb);
-
+void linphone_core_set_waiting_callback(LinphoneCore *lc, LinphoneWaitingCallback cb, void *user_context);
 
 /*returns the list of registered SipSetup (linphonecore plugins) */
 const MSList * linphone_core_get_sip_setups(LinphoneCore *lc);
