@@ -28,6 +28,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <unistd.h>
 #include <fcntl.h>
 
+#undef snprintf
 #include <ortp/stun.h>
 
 #ifndef WIN32
@@ -678,7 +679,9 @@ static int extract_sip_port(const char *config){
 	return ret;
 }
 
-int linphone_core_wake_up_possible_already_running_instance(const char *config_file){
+int linphone_core_wake_up_possible_already_running_instance(
+    const char * config_file, const char * addr_to_call)
+{
 	int port=extract_sip_port(config_file);
 	const char *wakeup="WAKEUP sip:127.0.0.1 SIP/2.0\r\n"
 		"Via: SIP/2.0/UDP 127.0.0.1:%i;rport;branch=z9hG4bK%u\r\n"
@@ -687,6 +690,15 @@ int linphone_core_wake_up_possible_already_running_instance(const char *config_f
 		"CSeq: 1 WAKEUP\r\n"
 		"Call-ID: %u@onsantape\r\n"
 		"Content-length: 0\r\n\r\n";
+	const char * call = "REFER sip:127.0.0.1 SIP/2.0\r\n"
+		"Via: SIP/2.0/UDP 127.0.0.1:%i;rport;branch=z9hG4bK%u\r\n"
+		"From: <sip:another_linphone@127.0.0.1>;tag=%u\r\n"
+		"To:   <sip:you@127.0.0.1>\r\n"
+		"Refer-To: %s\r\n"
+		"CSeq: 1 WAKEUP\r\n"
+		"Call-ID: %u@onsantape\r\n"
+		"Content-length: 0\r\n\r\n";
+
 	/*make sure ortp is initialized (it initializes win32 socket api)*/
 	ortp_init();
 	if (port>0){
@@ -700,7 +712,12 @@ int linphone_core_wake_up_possible_already_running_instance(const char *config_f
 			if (sock<0) sock=create_socket(++locport);
 			if (sock>=0){
 				char req[512];
-				snprintf(req,sizeof(req),wakeup,locport,random(),random(),random());
+				if (addr_to_call != NULL)
+					snprintf(req, sizeof(req), call, locport,
+			     		random(), random(), addr_to_call, random());
+				else
+					snprintf(req, sizeof(req), wakeup, locport,
+			     		random(), random(), random());
 				if (connect(sock,(struct sockaddr*)&ss,sslen)<0){
 					fprintf(stderr,"connect failed: %s\n",getSocketError());
 				}else if (send(sock,req,strlen(req),0)>0){
@@ -714,15 +731,69 @@ int linphone_core_wake_up_possible_already_running_instance(const char *config_f
 							break;
 						}
 #ifdef WIN32
-						Sleep(10);
+						Sleep(100);
 #else
-						usleep(10000);
+						usleep(100000);
 #endif
 					}
-				}else ms_message("sendto() of WAKEUP request failed, nobody to wakeup.");
+				}else{
+					ms_message("sendto() of WAKEUP request failed, nobody to wakeup.");
+				}
 			}
 			close_socket(sock);
 		}
 	}
 	return -1;
 }
+
+int linphone_core_get_local_ip_for(const char *dest, char *result){
+	int err,tmp;
+	struct addrinfo hints;
+	struct addrinfo *res=NULL;
+	struct sockaddr_storage addr;
+	ortp_socket_t sock;
+	socklen_t s;
+
+	memset(&hints,0,sizeof(hints));
+	hints.ai_family=PF_UNSPEC;
+	hints.ai_socktype=SOCK_DGRAM;
+	/*hints.ai_flags=AI_NUMERICHOST|AI_CANONNAME;*/
+	err=getaddrinfo(dest,"5060",&hints,&res);
+	if (err!=0){
+		ms_error("getaddrinfo() error: %s",gai_strerror(err));
+		return -1;
+	}
+	if (res==NULL){
+		ms_error("bug: getaddrinfo returned nothing.");
+		return -1;
+	}
+	sock=socket(res->ai_family,SOCK_DGRAM,0);
+	tmp=1;
+	err=setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,&tmp,sizeof(int));
+	if (err<0){
+		ms_warning("Error in setsockopt: %s",strerror(errno));
+	}
+	err=connect(sock,res->ai_addr,res->ai_addrlen);
+	if (err<0) {
+		ms_error("Error in connect: %s",strerror(errno));
+ 		freeaddrinfo(res);
+ 		close(sock);
+		return -1;
+	}
+	freeaddrinfo(res);
+	res=NULL;
+	s=sizeof(addr);
+	err=getsockname(sock,(struct sockaddr*)&addr,&s);
+	if (err!=0) {
+		ms_error("Error in getsockname: %s",strerror(errno));
+		close(sock);
+		return -1;
+	}
+	err=getnameinfo((struct sockaddr *)&addr,s,result,LINPHONE_IPADDR_SIZE,NULL,0,NI_NUMERICHOST);
+	if (err!=0){
+		ms_error("getnameinfo error: %s",strerror(errno));
+	}
+	close(sock);
+	ms_message("Local interface to reach %s is %s.",dest,result);
+	return 0;
+}
