@@ -54,35 +54,69 @@ void linphone_proxy_config_destroy(LinphoneProxyConfig *obj){
 	if (obj->ssctx!=NULL) sip_setup_context_free(obj->ssctx);
 	if (obj->realm!=NULL) ms_free(obj->realm);
 	if (obj->type!=NULL) ms_free(obj->type);
+	if (obj->contact_addr!=NULL) ms_free(obj->contact_addr);
 }
 
 bool_t linphone_proxy_config_is_registered(const LinphoneProxyConfig *obj){
 	return obj->registered;
 }
 
-void linphone_proxy_config_register_again_with_updated_contact(LinphoneProxyConfig *obj, osip_message_t *orig_request, osip_message_t *last_answer){
+void linphone_proxy_config_get_contact(LinphoneProxyConfig *cfg, const char **ip, int *port){
+	if (cfg->registered){
+		*ip=cfg->contact_addr;
+		*port=cfg->contact_port;
+	}else{
+		*ip=NULL;
+		*port=0;
+	}
+}
+
+static void update_contact(LinphoneProxyConfig *cfg, const char *ip, const char *port){
+	if (cfg->contact_addr){
+		ms_free(cfg->contact_addr);
+	}
+	cfg->contact_addr=ms_strdup(ip);
+	cfg->contact_port=atoi(port);
+	if (cfg->contact_port==0) cfg->contact_port=5060;
+}
+
+bool_t linphone_proxy_config_register_again_with_updated_contact(LinphoneProxyConfig *obj, osip_message_t *orig_request, osip_message_t *last_answer){
 	osip_message_t *msg;
 	const char *rport,*received;
 	osip_via_t *via=NULL;
 	osip_generic_param_t *param=NULL;
 	osip_contact_t *ctt=NULL;
 	osip_message_get_via(last_answer,0,&via);
-	if (!via) return;
+	if (!via) return FALSE;
 	osip_via_param_get_byname(via,"rport",&param);
 	if (param) rport=param->gvalue;
-	else return;
+	else return FALSE;
 	param=NULL;
 	osip_via_param_get_byname(via,"received",&param);
 	if (param) received=param->gvalue;
-	else return;
+	else return FALSE;
 	osip_message_get_contact(orig_request,0,&ctt);
-	if (strcmp(ctt->url->host,received)==0 && (ctt->url->port!=0 && strcmp(ctt->url->port,rport)==0)){
-		ms_message("Register has up to date contact, doing nothing.");
-		return;
+	if (strcmp(ctt->url->host,received)==0){
+		/*ip address matches, check ports*/
+		const char *contact_port=ctt->url->port;
+		const char *via_rport=rport;
+		if (via_rport==NULL || strlen(via_rport)>0)
+			via_rport="5060";
+		if (contact_port==NULL || strlen(contact_port)>0)
+			contact_port="5060";
+		if (strcmp(contact_port,via_rport)==0){
+			ms_message("Register has up to date contact, doing nothing.");
+			return FALSE;
+		}else ms_message("ports do not match, need to update the register (%s <> %s)", contact_port,via_rport);
 	}
 	eXosip_lock();
 	msg=NULL;
 	eXosip_register_build_register(obj->rid,obj->expires,&msg);
+	if (msg==NULL){
+		eXosip_unlock();
+		ms_warning("Fail to create a contact updated register.");
+		return FALSE;
+	}
 	osip_message_get_contact(msg,0,&ctt);
 	if (ctt->url->host!=NULL){
 		osip_free(ctt->url->host);
@@ -94,7 +128,9 @@ void linphone_proxy_config_register_again_with_updated_contact(LinphoneProxyConf
 	ctt->url->port=osip_strdup(rport);
 	eXosip_register_send_register(obj->rid,msg);
 	eXosip_unlock();
+	update_contact(obj,received,rport);
 	ms_message("Resending new register with updated contact %s:%s",received,rport);
+	return TRUE;
 }
 
 int linphone_proxy_config_set_server_addr(LinphoneProxyConfig *obj, const char *server_addr){
@@ -235,8 +271,10 @@ static void linphone_proxy_config_register(LinphoneProxyConfig *obj){
 	if (obj->reg_sendregister){
 		char *ct=NULL;
 		osip_message_t *msg=NULL;
+		eXosip_lock();
 		obj->rid=eXosip_register_build_initial_register(id_str,obj->reg_proxy,NULL,obj->expires,&msg);
 		eXosip_register_send_register(obj->rid,msg);
+		eXosip_unlock();
 		if (ct!=NULL) osip_free(ct);
 	}
 }
@@ -504,7 +542,7 @@ void linphone_proxy_config_process_authentication_failure(LinphoneCore *lc, eXos
 	LinphoneProxyConfig *cfg=linphone_core_get_proxy_config_from_rid(lc, ev->rid);
 	if (cfg){
 		cfg->auth_failures++;
-		/*restart a new register */
+		/*restart a new register so that the user gets a chance to be prompted for a password*/
 		if (cfg->auth_failures==1){
 			linphone_proxy_config_register(cfg);
 		}
@@ -634,6 +672,28 @@ SipSetupContext *linphone_proxy_config_get_sip_setup_context(LinphoneProxyConfig
 	return cfg->ssctx;
 }
 
+LinphoneAccountCreator *linphone_account_creator_new(struct _LinphoneCore *core, const char *type){
+	LinphoneAccountCreator *obj;
+	LinphoneProxyConfig *cfg;
+	SipSetup *ss=sip_setup_lookup(type);
+	SipSetupContext *ssctx;
+	if (!ss){
+		return NULL;
+	}
+	if (!(sip_setup_get_capabilities(ss) & SIP_SETUP_CAP_ACCOUNT_MANAGER)){
+		ms_error("%s cannot manage accounts.");
+		return NULL;
+	}
+	obj=ms_new0(LinphoneAccountCreator,1);
+	cfg=linphone_proxy_config_new();
+	ssctx=sip_setup_context_new(ss,cfg);
+	obj->lc=core;
+	obj->ssctx=ssctx;
+	set_string(&obj->domain,sip_setup_context_get_domains(ssctx)[0]);
+	cfg->lc=core;
+	return obj;
+}
+
 void linphone_account_creator_set_username(LinphoneAccountCreator *obj, const char *username){
 	set_string(&obj->username,username);
 }
@@ -646,7 +706,15 @@ void linphone_account_creator_set_domain(LinphoneAccountCreator *obj, const char
 	set_string(&obj->domain,domain);
 }
 
-int linphone_account_creator_test(LinphoneAccountCreator *obj){
+const char * linphone_account_creator_get_username(LinphoneAccountCreator *obj){
+	return obj->username;
+}
+
+const char * linphone_account_creator_get_domain(LinphoneAccountCreator *obj){
+	return obj->domain;
+}
+
+int linphone_account_creator_test_existence(LinphoneAccountCreator *obj){
 	SipSetupContext *ssctx=obj->ssctx;
 	char *uri=ms_strdup_printf("%s@%s",obj->username,obj->domain);
 	int err=sip_setup_context_account_exists(ssctx,uri);
