@@ -19,13 +19,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
-#else
-#define PACKAGE_DATA_DIR "./"
-
-#ifndef LINPHONE_VERSION
-#define LINPHONE_VERSION "3.1.2"
-#endif
-
 #endif
 
 #include <gtk/gtk.h>
@@ -40,7 +33,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define _(String) gettext (String)
 #endif
 
+#ifdef USE_BUILDDATE_VERSION
+#include "version_date.h"
+#undef LINPHONE_VERSION
+#define LINPHONE_VERSION LINPHONE_VERSION_DATE
+#endif
+
 GdkPixbuf * create_pixbuf(const gchar *filename);
+GdkPixbufAnimation *create_pixbuf_animation(const gchar *filename);
 void add_pixmap_directory(const gchar *directory);
 GtkWidget *linphone_gtk_create_window(const char *window_name);
 GtkWidget *linphone_gtk_get_widget(GtkWidget *window, const char *name);
@@ -48,6 +48,7 @@ LinphoneCore *linphone_gtk_get_core(void);
 GtkWidget *linphone_gtk_get_main_window();
 void linphone_gtk_display_something(GtkMessageType type,const gchar *message);
 void linphone_gtk_start_call(GtkWidget *button);
+void linphone_gtk_call_terminated();
 void linphone_gtk_show_friends(void);
 void linphone_gtk_show_contact(LinphoneFriend *lf);
 void linphone_gtk_set_my_presence(LinphoneOnlineStatus ss);
@@ -66,6 +67,16 @@ int linphone_gtk_get_ui_config_int(const char *key, int def);
 void linphone_gtk_open_browser(const char *url);
 void linphone_gtk_check_for_new_version(void);
 const char *linphone_gtk_get_lang(const char *config_file);
+void linphone_gtk_set_lang(const char *code);
 SipSetupContext* linphone_gtk_get_default_sip_setup_context(void);
 void linphone_gtk_show_buddy_lookup_window(SipSetupContext *ctx);
 void * linphone_gtk_wait(LinphoneCore *lc, void *ctx, LinphoneWaitingState ws, const char *purpose, float progress);
+gchar *linphone_gtk_get_display_name(const char *sip_uri);
+
+/*functions controlling the in-call view*/
+void linphone_gtk_show_in_call_view(void);
+void linphone_gtk_show_idle_view(void);
+void linphone_gtk_in_call_view_set_calling(const char *uri);
+void linphone_gtk_in_call_view_set_in_call(const char *uri);
+void linphone_gtk_in_call_view_update_duration(int duration);
+void linphone_gtk_in_call_view_terminate(const char *error_msg);
