@@ -19,9 +19,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #define USE_LIBGLADE 1
 
+#include "linphone.h"
 #include "lpconfig.h"
 
-#include "linphone.h"
+
 
 #ifdef USE_LIBGLADE
 #include <glade/glade.h>
@@ -31,14 +32,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <sys/stat.h>
 #include <unistd.h>
 
-#ifndef GETTEXT_PACKAGE
-#define GETTEXT_PACKAGE "linphone"
-#endif
-
-#ifndef PACKAGE_LOCALE_DIR
-#define PACKAGE_LOCALE_DIR "share/locale/"
-#endif
-
 #define LINPHONE_ICON "linphone2.png"
 
 const char *this_program_ident_string="linphone_ident_string=" LINPHONE_VERSION;
@@ -59,6 +52,8 @@ static void linphone_gtk_display_url(LinphoneCore *lc, const char *msg, const ch
 static void linphone_gtk_display_question(LinphoneCore *lc, const char *question);
 static void linphone_gtk_call_log_updated(LinphoneCore *lc, LinphoneCallLog *cl);
 static void linphone_gtk_general_state(LinphoneCore *lc, LinphoneGeneralState *gstate);
+static void linphone_gtk_refer_received(LinphoneCore *lc, const char *refer_to);
+static gboolean linphone_gtk_auto_answer(GtkWidget *incall_window);
 
 static LinphoneCoreVTable vtable={
 	.show=linphone_gtk_show,
@@ -74,28 +69,52 @@ static LinphoneCoreVTable vtable={
 	.display_question=linphone_gtk_display_question,
 	.call_log_updated=linphone_gtk_call_log_updated,
 	.text_received=linphone_gtk_text_received,
-	.general_state=linphone_gtk_general_state
+	.general_state=linphone_gtk_general_state,
+	.refer_received=linphone_gtk_refer_received
 };
 
 static gboolean verbose=0;
-static GOptionEntry linphone_options[2]={
+static gboolean auto_answer = 0;
+static gchar * addr_to_call = NULL;
+static GOptionEntry linphone_options[]={
 	{
 		.long_name="verbose",
 		.short_name= '\0',
 		.arg=G_OPTION_ARG_NONE,
 		.arg_data= (gpointer)&verbose,
-		.description="log to stdout some debug information while running."
-	}
+		.description=N_("log to stdout some debug information while running.")
+	},
+	{				/* zsd addition */
+	    .long_name = "call",
+	    .short_name = 'c',
+	    .arg = G_OPTION_ARG_STRING,
+	    .arg_data = &addr_to_call,
+	    .description = N_("address to call right now")
+	},
+	{				/* zsd addition */
+	    .long_name = "auto-answer",
+	    .short_name = 'a',
+	    .arg = G_OPTION_ARG_NONE,
+	    .arg_data = (gpointer) & auto_answer,
+	    .description = N_("if set automatically answer incoming calls")
+	},
+	{0}
 };
 
 #define INSTALLED_XML_DIR PACKAGE_DATA_DIR "/linphone"
 #define BUILD_TREE_XML_DIR "gtk-glade"
+
+#ifndef WIN32
 #define CONFIG_FILE ".linphonerc"
+#else
+#define CONFIG_FILE "linphonerc"
+#endif
+
+
 
 static char _config_file[1024];
 
 const char *linphone_gtk_get_config_file(){
-	const char *home;
 	/*try accessing a local file first if exists*/
 	if (access(CONFIG_FILE,F_OK)==0){
 		snprintf(_config_file,sizeof(_config_file),"%s",CONFIG_FILE);
@@ -103,13 +122,13 @@ const char *linphone_gtk_get_config_file(){
 #ifdef WIN32
 		const char *appdata=getenv("APPDATA");
 		if (appdata){
-			snprintf(_config_file,sizeof(_config_file),"%s\\%s",appdata,"Linphone\\");
+			snprintf(_config_file,sizeof(_config_file),"%s\\%s",appdata,LINPHONE_CONFIG_DIR);
 			CreateDirectory(_config_file,NULL);
-			snprintf(_config_file,sizeof(_config_file),"%s\\%s",appdata,"Linphone\\linphonerc");
+			snprintf(_config_file,sizeof(_config_file),"%s\\%s",appdata,LINPHONE_CONFIG_DIR "\\" CONFIG_FILE);
 		}
 #else
-		home=getenv("HOME");
-		if (home==NULL) home="";
+		const char *home=getenv("HOME");
+		if (home==NULL) home=".";
 		snprintf(_config_file,sizeof(_config_file),"%s/%s",home,CONFIG_FILE);
 #endif
 	}
@@ -119,7 +138,7 @@ const char *linphone_gtk_get_config_file(){
 static void linphone_gtk_init_liblinphone(const char *file){
 	linphone_core_set_user_agent("Linphone", LINPHONE_VERSION);
 	the_core=linphone_core_new(&vtable,file,NULL);
-	linphone_core_set_waiting_callback(the_core,linphone_gtk_wait);
+	linphone_core_set_waiting_callback(the_core,linphone_gtk_wait,NULL);
 }
 
 
@@ -283,12 +302,18 @@ void linphone_gtk_about_response(GtkDialog *dialog, gint id){
 	}
 }
 
+static void about_url_clicked(GtkAboutDialog *dialog, const char *url, gpointer data){
+	g_message("About url clicked");
+	linphone_gtk_open_browser(url);
+}
+
 void linphone_gtk_show_about(){
 	struct stat filestat;
-	const char *license_file=PACKAGE_DATA_DIR "/doc/COPYING";
+	const char *license_file=PACKAGE_DATA_DIR "/linphone/COPYING";
 	GtkWidget *about;
-
+	
 	about=linphone_gtk_create_window("about");
+	gtk_about_dialog_set_url_hook(about_url_clicked,NULL,NULL);
 	memset(&filestat,0,sizeof(filestat));
 	if (stat(license_file,&filestat)!=0){
 		license_file="COPYING";
@@ -304,7 +329,8 @@ void linphone_gtk_show_about(){
 		g_free(license);
 	}
 	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about),LINPHONE_VERSION);
-
+	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(about),linphone_gtk_get_ui_config("title","Linphone"));
+	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about),linphone_gtk_get_ui_config("home","http://www.linphone.org"));
 	gtk_widget_show(about);
 }
 
@@ -313,8 +339,17 @@ static void set_video_window_decorations(GdkWindow *w){
 	const char *icon_path=linphone_gtk_get_ui_config("icon","linphone2.png");
 	char video_title[256];
 	GdkPixbuf *pbuf=create_pixbuf(icon_path);
-	snprintf(video_title,sizeof(video_title),"%s video",title);
+	if (!linphone_core_in_call(linphone_gtk_get_core())){
+		snprintf(video_title,sizeof(video_title),"%s video",title);	
+	}else{
+		const char *uri=linphone_core_get_remote_uri(linphone_gtk_get_core());
+		gchar *display_name=linphone_gtk_get_display_name(uri);
+		snprintf(video_title,sizeof(video_title),"Call with %s",display_name);
+		g_free(display_name);
+	}
 	gdk_window_set_title(w,video_title);
+	/*gdk_window_set_urgency_hint(w,TRUE);*/
+	gdk_window_raise(w);
 	if (pbuf){
 		GList *l=NULL;
 		l=g_list_append(l,pbuf);
@@ -324,6 +359,11 @@ static void set_video_window_decorations(GdkWindow *w){
 	}
 }
 
+static gboolean video_needs_update=FALSE;
+
+static void update_video_title(){
+	video_needs_update=TRUE;
+}
 
 static gboolean linphone_gtk_iterate(LinphoneCore *lc){
 	unsigned long id;
@@ -335,19 +375,30 @@ static gboolean linphone_gtk_iterate(LinphoneCore *lc){
 	in_iterate=TRUE;
 	linphone_core_iterate(lc);
 	id=linphone_core_get_native_video_window_id(lc);
-	if (id!=previd){
-		ms_message("Updating window decorations");
+	if (id!=previd || video_needs_update){
 		GdkWindow *w;
 		previd=id;
 		if (id!=0){
+			ms_message("Updating window decorations");
+#ifndef WIN32
 			w=gdk_window_foreign_new(id);
+#else
+			w=gdk_window_foreign_new((HANDLE)id);
+#endif
 			if (w) {
 				set_video_window_decorations(w);
 				g_object_unref(G_OBJECT(w));
 			}
 			else ms_error("gdk_window_foreign_new() failed");
+			if (video_needs_update) video_needs_update=FALSE;
 		}
 	}
+	if (addr_to_call!=NULL){
+		GtkWidget *uri_bar=linphone_gtk_get_widget(linphone_gtk_get_main_window(),"uribar");
+		gtk_entry_set_text(GTK_ENTRY(uri_bar),addr_to_call);
+		addr_to_call=NULL;
+		linphone_gtk_start_call(uri_bar);	
+	}
 	in_iterate=FALSE;
 	return TRUE;
 }
@@ -424,37 +475,56 @@ static void completion_add_text(GtkEntry *entry, const char *text){
 	save_uri_history();
 }
 
-static void linphone_gtk_call_terminated(GtkWidget *mw){
-	gtk_widget_hide(linphone_gtk_get_widget(mw,"terminate_call"));
-	gtk_widget_show(linphone_gtk_get_widget(mw,"start_call"));
+void linphone_gtk_call_terminated(const char *error){
+	GtkWidget *mw=linphone_gtk_get_main_window();
+	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"terminate_call"),FALSE);
+	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"start_call"),TRUE);
+	gtk_widget_hide_all(linphone_gtk_get_widget(mw,"go_to_call_view_box"));
+	linphone_gtk_in_call_view_terminate(error);
+	update_video_title();
 	g_object_set_data(G_OBJECT(mw),"incoming_call",NULL);
+	
 }
 
-gboolean check_call_active(){
-	if (!linphone_core_in_call(linphone_gtk_get_core())){
-		linphone_gtk_call_terminated(linphone_gtk_get_main_window());
-		return FALSE;
+static gboolean in_call_timer(){
+	if (linphone_core_in_call(linphone_gtk_get_core())){
+		linphone_gtk_in_call_view_update_duration(
+			linphone_core_get_current_call_duration(linphone_gtk_get_core()));
+		return TRUE;
 	}
-	return TRUE;
+	return FALSE;
 }
 
 static void linphone_gtk_call_started(GtkWidget *mw){
-	gtk_widget_hide(linphone_gtk_get_widget(mw,"start_call"));
-	gtk_widget_show(linphone_gtk_get_widget(mw,"terminate_call"));
-	g_timeout_add(250,(GSourceFunc)check_call_active,NULL);
+	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"start_call"),FALSE);
+	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"terminate_call"),TRUE);
+	gtk_widget_show_all(linphone_gtk_get_widget(mw,"go_to_call_view_box"));
+	update_video_title();
+	g_timeout_add(250,(GSourceFunc)in_call_timer,NULL);
+}
+
+static gboolean linphone_gtk_start_call_do(GtkWidget *uri_bar){
+	const char *entered=gtk_entry_get_text(GTK_ENTRY(uri_bar));
+	if (linphone_core_invite(linphone_gtk_get_core(),entered)==0) {
+		completion_add_text(GTK_ENTRY(uri_bar),entered);
+	}else{
+	}
+	return FALSE;
 }
 
 void linphone_gtk_start_call(GtkWidget *w){
 	LinphoneCore *lc=linphone_gtk_get_core();
-	if (linphone_core_inc_invite_pending(lc)){
+	if (linphone_core_inc_invite_pending(lc) || linphone_core_in_call(lc)) {
 		/*already in call */
 	}else{
-		GtkWidget *uri_bar=linphone_gtk_get_widget(gtk_widget_get_toplevel(w),"uribar");
+		/*change into in-call mode, then do the work later as it might block a bit */
+		GtkWidget *mw=gtk_widget_get_toplevel(w);
+		GtkWidget *uri_bar=linphone_gtk_get_widget(mw,"uribar");
 		const char *entered=gtk_entry_get_text(GTK_ENTRY(uri_bar));
-		if (linphone_core_invite(lc,entered)==0) {
-			linphone_gtk_call_started(linphone_gtk_get_main_window());
-			completion_add_text(GTK_ENTRY(uri_bar),entered);
-		}
+		linphone_gtk_call_started(mw);
+		linphone_gtk_in_call_view_set_calling(entered);
+		linphone_gtk_show_in_call_view();
+		g_timeout_add(100,(GSourceFunc)linphone_gtk_start_call_do,uri_bar);
 	}
 }
 
@@ -465,20 +535,29 @@ void linphone_gtk_uri_bar_activate(GtkWidget *w){
 
 void linphone_gtk_terminate_call(GtkWidget *button){
 	linphone_core_terminate_call(linphone_gtk_get_core(),NULL);
-	linphone_gtk_call_terminated(gtk_widget_get_toplevel(button));
 }
 
 void linphone_gtk_decline_call(GtkWidget *button){
 	linphone_core_terminate_call(linphone_gtk_get_core(),NULL);
-	linphone_gtk_call_terminated(linphone_gtk_get_main_window());
+	/* zsd note: there was a big here in 3.0.0 which caused an abort if
+	 * someone clicked "decline"... the following line of code looks
+	 * like a fix for that. */
 	gtk_widget_destroy(gtk_widget_get_toplevel(button));
 }
 
 void linphone_gtk_accept_call(GtkWidget *button){
-	linphone_core_accept_call(linphone_gtk_get_core(),NULL);
+	LinphoneCore *lc=linphone_gtk_get_core();
+	linphone_core_accept_call(lc,NULL);
 	g_object_set_data(G_OBJECT(linphone_gtk_get_main_window()),"incoming_call",NULL);
 	gtk_widget_destroy(gtk_widget_get_toplevel(button));
 	linphone_gtk_call_started(linphone_gtk_get_main_window());
+	linphone_gtk_in_call_view_set_in_call(linphone_core_get_remote_uri(lc));
+	linphone_gtk_show_in_call_view();
+}
+
+static gboolean linphone_gtk_auto_answer(GtkWidget *incall_window){
+	linphone_gtk_accept_call(linphone_gtk_get_widget(incall_window,"accept_call"));
+	return FALSE;
 }
 
 void linphone_gtk_set_audio_video(){
@@ -519,6 +598,11 @@ static void linphone_gtk_inv_recv(LinphoneCore *lc, const char *from){
 	GtkWidget *w=linphone_gtk_create_window("incoming_call");
 	GtkWidget *label;
 	gchar *msg;
+
+	if (auto_answer){
+		g_timeout_add(2000,(GSourceFunc)linphone_gtk_auto_answer,w);
+	}
+
 	gtk_window_set_transient_for(GTK_WINDOW(w),GTK_WINDOW(linphone_gtk_get_main_window()));
 	gtk_window_set_position(GTK_WINDOW(w),GTK_WIN_POS_CENTER_ON_PARENT);
 
@@ -540,7 +624,6 @@ static void linphone_gtk_bye_recv(LinphoneCore *lc, const char *from){
 	if (icw!=NULL){
 		gtk_widget_destroy(icw);
 	}
-	linphone_gtk_call_terminated(linphone_gtk_get_main_window());
 }
 
 static void linphone_gtk_notify_recv(LinphoneCore *lc, LinphoneFriend * fid, const char *url, const char *status, const char *img){
@@ -672,6 +755,20 @@ static void linphone_gtk_call_log_updated(LinphoneCore *lc, LinphoneCallLog *cl)
 }
 
 static void linphone_gtk_general_state(LinphoneCore *lc, LinphoneGeneralState *gstate){
+	switch(gstate->new_state){
+		case GSTATE_CALL_OUT_CONNECTED:
+		case GSTATE_CALL_IN_CONNECTED:
+			linphone_gtk_in_call_view_set_in_call(linphone_core_get_remote_uri(lc));
+		break;
+		case GSTATE_CALL_ERROR:
+			linphone_gtk_call_terminated(gstate->message);
+		break;
+		case GSTATE_CALL_END:
+			linphone_gtk_call_terminated(NULL);
+		break;
+		default:
+		break;
+	}
 }
 
 static void icon_popup_menu(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data){
@@ -680,6 +777,8 @@ static void icon_popup_menu(GtkStatusIcon *status_icon, guint button, guint acti
 }
 
 void linphone_gtk_open_browser(const char *url){
+	gtk_show_uri(NULL,url,GDK_CURRENT_TIME,NULL);
+#if 0
 #ifdef WIN32
 	ShellExecute(0,"open",url,NULL,NULL,1);
 #else
@@ -687,6 +786,7 @@ void linphone_gtk_open_browser(const char *url){
 	snprintf(cl,sizeof(cl),"/usr/bin/x-www-browser %s",url);
 	g_spawn_command_line_async(cl,NULL);
 #endif
+#endif
 }
 
 void linphone_gtk_link_to_website(GtkWidget *item){
@@ -774,7 +874,13 @@ void linphone_gtk_load_identities(void){
 
 static void linphone_gtk_dtmf_clicked(GtkButton *button){
 	const char *label=gtk_button_get_label(button);
-	linphone_core_send_dtmf(linphone_gtk_get_core(),label[0]);
+	if (linphone_core_in_call(linphone_gtk_get_core())){
+		linphone_core_send_dtmf(linphone_gtk_get_core(),label[0]);
+	}else{
+		GtkWidget *uri_bar=linphone_gtk_get_widget(gtk_widget_get_toplevel(GTK_WIDGET(button)),"uribar");
+		int pos=-1;
+		gtk_editable_insert_text(GTK_EDITABLE(uri_bar),label,1,&pos);
+	}
 }
 
 static void linphone_gtk_connect_digits(void){
@@ -814,10 +920,10 @@ static void linphone_gtk_configure_main_window(){
 	static const char *stop_call_icon;
 	GtkWidget *w=linphone_gtk_get_main_window();
 	if (!config_loaded){
-		title=linphone_gtk_get_ui_config("title",NULL);
+		title=linphone_gtk_get_ui_config("title","Linphone");
 		home=linphone_gtk_get_ui_config("home","http://www.linphone.org");
-		start_call_icon=linphone_gtk_get_ui_config("start_call_icon",NULL);
-		stop_call_icon=linphone_gtk_get_ui_config("stop_call_icon",NULL);
+		start_call_icon=linphone_gtk_get_ui_config("start_call_icon","green.png");
+		stop_call_icon=linphone_gtk_get_ui_config("stop_call_icon","red.png");
 		config_loaded=TRUE;
 	}
 	linphone_gtk_configure_window(w,"main_window");
@@ -835,6 +941,7 @@ static void linphone_gtk_configure_main_window(){
 	if (stop_call_icon){
 		GdkPixbuf *pbuf=create_pixbuf(stop_call_icon);
 		gtk_image_set_from_pixbuf(GTK_IMAGE(linphone_gtk_get_widget(w,"terminate_call_icon")),pbuf);
+		gtk_image_set_from_pixbuf(GTK_IMAGE(linphone_gtk_get_widget(w,"in_call_terminate_icon")),pbuf);
 		g_object_unref(G_OBJECT(pbuf));
 	}
 	if (home){
@@ -868,7 +975,7 @@ void linphone_gtk_close(){
 	/*shutdown call if any*/
 	if (linphone_core_in_call(lc)){
 		linphone_core_terminate_call(lc,NULL);
-		linphone_gtk_call_terminated(the_ui);
+		linphone_gtk_call_terminated(NULL);
 	}
 	linphone_core_enable_video_preview(lc,FALSE);
 }
@@ -912,6 +1019,16 @@ void linphone_gtk_log_handler(OrtpLogLevel lev, const char *fmt, va_list args){
 	linphone_gtk_log_push(lev,fmt,args);
 }
 
+
+static void linphone_gtk_refer_received(LinphoneCore *lc, const char *refer_to){
+    GtkEntry * uri_bar =GTK_ENTRY(linphone_gtk_get_widget(
+		linphone_gtk_get_main_window(), "uribar"));
+	linphone_gtk_show_main_window();
+	gtk_entry_set_text(uri_bar, refer_to);
+	linphone_gtk_start_call(linphone_gtk_get_main_window());
+}
+
+
 int main(int argc, char *argv[]){
 #ifdef ENABLE_NLS
 	void *p;
@@ -923,11 +1040,7 @@ int main(int argc, char *argv[]){
 	gdk_threads_init();
 	
 	config_file=linphone_gtk_get_config_file();
-	if (linphone_core_wake_up_possible_already_running_instance(config_file)==0){
-		g_warning("Another running instance of linphone has been detected. It has been woken-up.");
-		g_warning("This instance is going to exit now.");
-		return 0;
-	}
+
 #ifdef WIN32
 	/*workaround for windows: sometimes LANG is defined to an integer value, not understood by gtk */
 	if ((lang=getenv("LANG"))!=NULL){
@@ -961,17 +1074,23 @@ int main(int argc, char *argv[]){
 	gtk_rc_add_default_file("./gtkrc");
 #endif
 	gdk_threads_enter();
+	
 	if (!gtk_init_with_args(&argc,&argv,_("A free SIP video-phone"),
 				linphone_options,NULL,NULL)){
 		gdk_threads_leave();
 		return -1;
 	}
-	
+	if (linphone_core_wake_up_possible_already_running_instance(
+		config_file, addr_to_call) == 0){
+		g_message("addr_to_call=%s",addr_to_call);
+		g_warning("Another running instance of linphone has been detected. It has been woken-up.");
+		g_warning("This instance is going to exit now.");
+		gdk_threads_leave();
+		return 0;
+	}
+
 	add_pixmap_directory("pixmaps");
 	add_pixmap_directory(PACKAGE_DATA_DIR "/pixmaps/linphone");
-#ifdef WIN32
-	add_pixmap_directory("linphone");
-#endif
 
 	the_ui=linphone_gtk_create_window("main");
 	
@@ -986,6 +1105,7 @@ int main(int argc, char *argv[]){
 	linphone_gtk_init_status_icon();
 	linphone_gtk_show_main_window();
 	linphone_gtk_check_for_new_version();
+
 	gtk_main();
 	gdk_threads_leave();
 	linphone_gtk_destroy_log_window();
@@ -994,5 +1114,3 @@ int main(int argc, char *argv[]){
 	gtk_status_icon_set_visible(icon,FALSE);
 	return 0;
 }
-
-
